@@ -1,6 +1,7 @@
 # type: ignore
 
 from collections import namedtuple
+import csv
 import os
 import shutil
 from tempfile import NamedTemporaryFile
@@ -10,8 +11,9 @@ import click
 from click.testing import CliRunner
 
 from mycli.main import MyCli, cli, thanks_picker
+import mycli.packages.special
 from mycli.packages.special.main import COMMANDS as SPECIAL_COMMANDS
-from mycli.sqlexecute import ServerInfo
+from mycli.sqlexecute import ServerInfo, SQLExecute
 from test.utils import HOST, PASSWORD, PORT, USER, dbtest, run
 
 test_dir = os.path.abspath(os.path.dirname(__file__))
@@ -37,6 +39,245 @@ CLI_ARGS = [
 ]
 
 
+@dbtest
+def test_ssl_mode_on(executor, capsys):
+    runner = CliRunner()
+    ssl_mode = "on"
+    sql = "select * from performance_schema.session_status where variable_name = 'Ssl_cipher'"
+    result = runner.invoke(cli, args=CLI_ARGS + ["--csv", "--ssl-mode", ssl_mode], input=sql)
+    result_dict = next(csv.DictReader(result.stdout.split("\n")))
+    ssl_cipher = result_dict["VARIABLE_VALUE"]
+    assert ssl_cipher
+
+
+@dbtest
+def test_ssl_mode_auto(executor, capsys):
+    runner = CliRunner()
+    ssl_mode = "auto"
+    sql = "select * from performance_schema.session_status where variable_name = 'Ssl_cipher'"
+    result = runner.invoke(cli, args=CLI_ARGS + ["--csv", "--ssl-mode", ssl_mode], input=sql)
+    result_dict = next(csv.DictReader(result.stdout.split("\n")))
+    ssl_cipher = result_dict["VARIABLE_VALUE"]
+    assert ssl_cipher
+
+
+@dbtest
+def test_ssl_mode_off(executor, capsys):
+    runner = CliRunner()
+    ssl_mode = "off"
+    sql = "select * from performance_schema.session_status where variable_name = 'Ssl_cipher'"
+    result = runner.invoke(cli, args=CLI_ARGS + ["--csv", "--ssl-mode", ssl_mode], input=sql)
+    result_dict = next(csv.DictReader(result.stdout.split("\n")))
+    ssl_cipher = result_dict["VARIABLE_VALUE"]
+    assert not ssl_cipher
+
+
+@dbtest
+def test_ssl_mode_overrides_ssl(executor, capsys):
+    runner = CliRunner()
+    ssl_mode = "off"
+    sql = "select * from performance_schema.session_status where variable_name = 'Ssl_cipher'"
+    result = runner.invoke(cli, args=CLI_ARGS + ["--csv", "--ssl-mode", ssl_mode, "--ssl"], input=sql)
+    result_dict = next(csv.DictReader(result.stdout.split("\n")))
+    ssl_cipher = result_dict["VARIABLE_VALUE"]
+    assert not ssl_cipher
+
+
+@dbtest
+def test_ssl_mode_overrides_no_ssl(executor, capsys):
+    runner = CliRunner()
+    ssl_mode = "on"
+    sql = "select * from performance_schema.session_status where variable_name = 'Ssl_cipher'"
+    result = runner.invoke(cli, args=CLI_ARGS + ["--csv", "--ssl-mode", ssl_mode, "--no-ssl"], input=sql)
+    result_dict = next(csv.DictReader(result.stdout.split("\n")))
+    ssl_cipher = result_dict["VARIABLE_VALUE"]
+    assert ssl_cipher
+
+
+@dbtest
+def test_reconnect_no_database(executor, capsys):
+    m = MyCli()
+    m.register_special_commands()
+    m.sqlexecute = SQLExecute(
+        None,
+        USER,
+        PASSWORD,
+        HOST,
+        PORT,
+        None,
+        None,
+        None,
+        None,
+        None,
+        None,
+        None,
+        None,
+        None,
+        None,
+    )
+    sql = "\\r"
+    result = next(mycli.packages.special.execute(executor, sql))
+    stdout, _stderr = capsys.readouterr()
+    assert result[-1] is None
+    assert "Already connected" in stdout
+
+
+@dbtest
+def test_reconnect_with_different_database(executor):
+    m = MyCli()
+    m.register_special_commands()
+    m.sqlexecute = SQLExecute(
+        None,
+        USER,
+        PASSWORD,
+        HOST,
+        PORT,
+        None,
+        None,
+        None,
+        None,
+        None,
+        None,
+        None,
+        None,
+        None,
+        None,
+    )
+    database_1 = "mycli_test_db"
+    database_2 = "mysql"
+    sql_1 = f"use {database_1}"
+    sql_2 = f"\\r {database_2}"
+    _result_1 = next(mycli.packages.special.execute(executor, sql_1))
+    result_2 = next(mycli.packages.special.execute(executor, sql_2))
+    expected = f'You are now connected to database "{database_2}" as user "{USER}"'
+    assert expected in result_2[-1]
+
+
+@dbtest
+def test_reconnect_with_same_database(executor):
+    m = MyCli()
+    m.register_special_commands()
+    m.sqlexecute = SQLExecute(
+        None,
+        USER,
+        PASSWORD,
+        HOST,
+        PORT,
+        None,
+        None,
+        None,
+        None,
+        None,
+        None,
+        None,
+        None,
+        None,
+        None,
+    )
+    database = "mysql"
+    sql = f"\\u {database}"
+    result = next(mycli.packages.special.execute(executor, sql))
+    sql = f"\\r {database}"
+    result = next(mycli.packages.special.execute(executor, sql))
+    expected = f'You are already connected to database "{database}" as user "{USER}"'
+    assert expected in result[-1]
+
+
+@dbtest
+def test_prompt_no_host_only_socket(executor):
+    mycli = MyCli()
+    mycli.prompt_format = "\\t \\u@\\h:\\d> "
+    mycli.sqlexecute = SQLExecute
+    mycli.sqlexecute.server_info = ServerInfo.from_version_string("8.0.44-0ubuntu0.24.04.1")
+    mycli.sqlexecute.host = None
+    mycli.sqlexecute.socket = "/var/run/mysqld/mysqld.sock"
+    mycli.sqlexecute.user = "root"
+    mycli.sqlexecute.dbname = "mysql"
+    mycli.sqlexecute.port = "3306"
+    prompt = mycli.get_prompt(mycli.prompt_format)
+    assert prompt == "MySQL root@localhost:mysql> "
+
+
+@dbtest
+def test_enable_show_warnings(executor):
+    mycli = MyCli()
+    mycli.register_special_commands()
+    sql = "\\W"
+    result = run(executor, sql)
+    assert result[0]["status"] == "Show warnings enabled."
+
+
+@dbtest
+def test_disable_show_warnings(executor):
+    mycli = MyCli()
+    mycli.register_special_commands()
+    sql = "\\w"
+    result = run(executor, sql)
+    assert result[0]["status"] == "Show warnings disabled."
+
+
+@dbtest
+def test_output_ddl_with_warning_and_show_warnings_enabled(executor):
+    runner = CliRunner()
+    db = "mycli_test_db"
+    table = "table_that_definitely_does_not_exist_1234"
+    sql = f"DROP TABLE IF EXISTS {db}.{table}"
+    result = runner.invoke(cli, args=CLI_ARGS + ["--show-warnings", "--no-warn"], input=sql)
+    expected = "Level\tCode\tMessage\nNote\t1051\tUnknown table 'mycli_test_db.table_that_definitely_does_not_exist_1234'\n"
+    assert expected in result.output
+
+
+@dbtest
+def test_output_with_warning_and_show_warnings_enabled(executor):
+    runner = CliRunner()
+    sql = "SELECT 1 + '0 foo'"
+    result = runner.invoke(cli, args=CLI_ARGS + ["--show-warnings"], input=sql)
+    expected = "1 + '0 foo'\n1.0\nLevel\tCode\tMessage\nWarning\t1292\tTruncated incorrect DOUBLE value: '0 foo'\n"
+    assert expected in result.output
+
+
+@dbtest
+def test_output_with_warning_and_show_warnings_disabled(executor):
+    runner = CliRunner()
+    sql = "SELECT 1 + '0 foo'"
+    result = runner.invoke(cli, args=CLI_ARGS + ["--no-show-warnings"], input=sql)
+    expected = "1 + '0 foo'\n1.0\nLevel\tCode\tMessage\nWarning\t1292\tTruncated incorrect DOUBLE value: '0 foo'\n"
+    assert expected not in result.output
+
+
+@dbtest
+def test_output_with_multiple_warnings_in_single_statement(executor):
+    runner = CliRunner()
+    sql = "SELECT 1 + '0 foo', 2 + '0 foo'"
+    result = runner.invoke(cli, args=CLI_ARGS + ["--show-warnings"], input=sql)
+    expected = (
+        "1 + '0 foo'\t2 + '0 foo'\n"
+        "1.0\t2.0\n"
+        "Level\tCode\tMessage\n"
+        "Warning\t1292\tTruncated incorrect DOUBLE value: '0 foo'\n"
+        "Warning\t1292\tTruncated incorrect DOUBLE value: '0 foo'\n"
+    )
+    assert expected in result.output
+
+
+@dbtest
+def test_output_with_multiple_warnings_in_multiple_statements(executor):
+    runner = CliRunner()
+    sql = "SELECT 1 + '0 foo'; SELECT 2 + '0 foo'"
+    result = runner.invoke(cli, args=CLI_ARGS + ["--show-warnings"], input=sql)
+    expected = (
+        "1 + '0 foo'\n"
+        "1.0\n"
+        "Level\tCode\tMessage\n"
+        "Warning\t1292\tTruncated incorrect DOUBLE value: '0 foo'\n"
+        "2 + '0 foo'\n"
+        "2.0\n"
+        "Level\tCode\tMessage\n"
+        "Warning\t1292\tTruncated incorrect DOUBLE value: '0 foo'\n"
+    )
+    assert expected in result.output
+
+
 @dbtest
 def test_execute_arg(executor):
     run(executor, "create table test (a text)")
@@ -324,6 +565,7 @@ def test_dsn(monkeypatch):
             self.destructive_warning = False
             self.main_formatter = Formatter()
             self.redirect_formatter = Formatter()
+            self.ssl_mode = "auto"
 
         def connect(self, **args):
             MockMyCli.connect_args = args
@@ -488,6 +730,7 @@ def test_ssh_config(monkeypatch):
             self.destructive_warning = False
             self.main_formatter = Formatter()
             self.redirect_formatter = Formatter()
+            self.ssl_mode = "auto"
 
         def connect(self, **args):
             MockMyCli.connect_args = args
